function [FI1,RE1,FI2,RE2,FI3,RE3]=kaskade(SM1,SP1,SM2,SP2,SM3,SP3)
%KASKADE Optimierung einer Kaskadenstruktur mit digitalen Reglern
%        [FI1,RE1,FI2,RE2,FI3,RE3]=kaskade(SM1,SP1,SM2,SP2,SM3,SP3)
% Eingnge:
% SPx      - Regelstrecken im Laplace-Bereich in >>>LTI<<< Syntax mit Totzeiten
%            ber Variable InputDelay
% SMx      - Steuermatrix als >>>'structure array'<<<
%            SMx.Optimierung={'bod','eez'} - oder m-Files gleicher Aufrufsyntax
%            SMx.OhneMenue  =Steuervektor zur Reglerberechnung ohne Menfhrung;
%                            vergleiche inp2(LTI) fr bod.m und eez.m
%            SMx.RegelKreis =[T m kode]=Steuervektor zur Regelkreisbeschreibung;
%                            vergleiche inp2,inp3,inp4(LTI) fr trans.m
% Ausgnge:
% FIx, REx - Digitale Filter bzw. Regler im Z-Bereich in >>>LTI<<< Syntax
% HINWEISE zu SMx.RegelKreis:
%          T:    gleiche Abtastzeiten in dieser Version     
%          m:    Totzeiten der SPx addieren sich zum jeweiligen m der Schleife
%          kode: == {0, 3} nur fr innere Schleife sinnvoll zulssig
% ACHTUNG  In Abhngigkeit von den Streckenparametern kann der Aufruf von minreal 
%          mit weiter verringerter Toleranz notwendig werden: X=minreal(X,Tol)

% SCHLSSELWORTE: Digitales Betragsoptimum, Digitaler Regler von PID-Typ,
%                 Endliche Einstellzeit, Regelschleife, Fhrungsgrenfilter, 
%		          Stroptimierung, Fhrungsoptimierung, Symmetrisches Optimum, 
%		          Steuervektor, Abschaltung Menfhrung, Z-Transformation,
%		          Totzeit, Polkompensation wahlweise, LTI

%  ***** Voraussetzungen: Vorhandensein von trans, bod, bodg, vobo, eez ****
%  Copyright (c) 1997-2007. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. ET,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_1.htm
%  Literatur zu Z-Transformation, Digitalem Betragsoptimum und Endlicher Einstellzeit: 
%                     Geitner, G.-H.:
%                     Entwurf digitaler Regler fr elektrische Antriebe.
%                     VDE-Verlag GmbH Berlin und Offenbach 1996
%                     ISBN 3-8007-1847-2

FT01='Falsche Anzahl Eingangs- und/oder Ausgangsargumente!';
FT02='Mindestens ein Eingangsargument SMx ist nicht vom Typ >>>STRUCT<<<! ';
FT03='Mindestens ein Eingangsargument SPx ist nicht vom Typ >>>LTI<<< !';
FT04='Ungleiche Abtastzeiten in dieser Version unzulssig!';
%nein: FT05='Nur fr innere Schleife ist kode={0, 3} sinnvoll!';
TOL=sqrt(1e-12);                               %Herabsetzung der Standardtoleranz

Anz=nargin/2; x1=abs('FI0'); x2=abs('RE0'); x4=abs('SM0'); x5=abs('SP0');
if nargin~=nargout|Anz<1|Anz>3|rem(Anz,1)~=0 error(FT01); end
for i=1:Anz
   if     ~isa(eval(setstr(x4+[0 0 i])),'struct') error(FT02);
   elseif ~isa(eval(setstr(x5+[0 0 i])),'lti')    error(FT03); end; end
x41=abs('SM0.Optimierung'); x42=abs('SM0.OhneMenue'); x43=abs('SM0.RegelKreis');
for i=1:Anz
   eval(['TVL=' x43+[0 0 i 0 0 0 0 0 0 0 0 0 0 0] ';']);   %TransformationsVektor
   T=TVL(1); m=TVL(2); kode=TVL(3);                        %Lokal
   eval(['SPL=' x5+[0 0 i] ';']); %P-Bereich: "Zuwachs" um lokalen Streckenanteil  
   if i==1                                              % >>>Innerste Schleife<<<
           SPG=SPL;                               %P-Bereich: lokale Gesamtstrecke
           SZd=tf(1,1);                           %Offener Kreis digitaler Anteil
           TVS=TVL;                               %Transformationsvektor Speicher
   else                                           %   >>> berlagerte Schleifen<<<
        SPG=series(SPG,SPL); SPG=minreal(SPG,TOL);%P-Bereich: lokale Gesamtstrecke
        if TVS(1)~=T error(FT04); end          %gleiche Abtastzeit in dieser Vers.
  %nein: if kode==0|kode==3 error(FT05); end  %berlagerte Schleifen: kode={1, 2}!
     if i==2                                             % >>>Mittlere Schleife<<<
        RF1=feedback(RE1,Zi); RF1=minreal(RF1,TOL);   %Zi von vorheriger Schleife!
        SZd=series(FI1,RF1); SZd=minreal(SZd,TOL); %Offener Kreis digitaler Anteil
     else                                                  % >>>uere Schleife<<<
        VWZ=series(RE2,SZd); VWZ=minreal(VWZ,TOL);   %SZd von vorheriger Schleife!
        RF2=feedback(VWZ,Zi); RF2=minreal(RF2,TOL);   %Zi von vorheriger Schleife!
        SZd=series(FI2,RF2); SZd=minreal(SZd,TOL); %Offener Kreis digitaler Anteil
     end
   end
   Zi=trans(SPG,T,m,kode);      %Transformation lokaler Gesamtstrecke im P-Bereich
   SZG=series(SZd,Zi); SZG=minreal(SZG,TOL);             %Z-Bereich: Gesamtstrecke 
                                                         %im offenen Kreis  
   s3=eval(setstr(x41+[0 0 i 0 0 0 0 0 0 0 0 0 0 0 0]));    %Optimierungsverfahren
   SV=eval(setstr(x42+[0 0 i 0 0 0 0 0 0 0 0 0 0]));                 %Steuervektor
   eval(['[' x2+[0 0 i] ',' x1+[0 0 i] ']=' s3 '(SZG,SV);']);         %Optimierung
end
