function [erg,z1,z2]=zust_au2(T,Tei,TM,TF,TL,SV)
%ZUST_AU2 Zustandsregelstruktur 1; 3 Koeff.; automatisiert; hier nur Strecke
%         [erg(,z1,z2]=zust_au2(T,Tei,TM,TF,TL(,SV)) - Einsatz des Digitalen 
%         Betragsoptimums (BOD) zur Berechnung der Koeffizienten einer 
%         digitalen Zustandsregelstruktur fuer ein Beispiel nach Brandenburg
%                                       at 35(1987)7, S. 275/83,
%         dort: Tei=4; TM=30; TF=5; und TL={30; 60; 120}
%      **** automatisierte Berechnung ****
%      T:	   Abtastzeit der digitalen Regelung in ms
%      Tei:	   Restzeitkonstante  (Verzoegerungsglied fuer Ggstrom) in ms
%      TM:	   Motorzeitkonstante (Integrator) in ms
%      TF:	   Federzeitkonstante (Integrator) in ms
%      TL:	   Lastzeitkonstante  (Integrator) in ms
%      (,SV):  wahlweise: Steuervektor zur Ablaufkontrolle, default SV==[1 0 0 0]
%                Ele1: Auswahl Berechnungsspalte in BM, hier={1,2,3}, default=1;
%                Bit2: Anzeige "removed states": 0=nein/1=ja; default=0;
%                Bit3: Teilergebnisanzeige: 0=nein/1=ja; default=0;
%                Bit4: Display & Diagnose fr fsolve: 0=nein/1=ja; default=0;
%      erg:	   [Kdelta Km Kn]
%      z1,z2:  siehe fsolve (out2,out3)

% SCHLUESSELWORTE: Digitales Betragsoptimum, Zustandsregelstruktur, elastisches
%                  Antriebssystem, Fuehrungsoptimierung, Stoeroptimierung

%  ***** Voraussetzungen: Vorhandensein von zust_aug2, minreal2 ****
%  Copyright (c) 1997-2007. All Rights Reserved. 
%                     Dr. Gert-Helge Geitner; TU Dresden, Fak. ET,
%                     Elektrotechnisches Institut (ETI); Mommsenstr. 13;
%                     D-01062 Dresden, Germany;
%                     http://eeiwzg.et.tu-dresden.de/ae2_files/ae_1.htm
%  Literatur zum Digitalen Betragsoptimum: Geitner, G.-H.:
%                     Entwurf digitaler Regler fuer elektrische Antriebe.
%                     VDE-Verlag GmbH Berlin und Offenbach 1995

error(nargchk(5,6,nargin))
if   nargin==5 SV=[1 0 0 0]; %default Werte fr Steuervektor SV
else [x,y]=size(SV);
     if     x~=1|y~=4 error('SV - falsche Dimension');     
     elseif ~(SV(1)==1|SV(1)==2|SV(1)==3) error('SV - Berechnungsspalte falsch!');
     elseif ~(SV(2)==0|SV(2)==1) error('SV - Schalter "removed states" falsch!');
     elseif ~(SV(3)==0|SV(3)==1) error('SV - Schalter Teilergebnisanzeige falsch!');
     elseif ~(SV(4)==0|SV(4)==1) error('SV - Schalter Optionsauswahl falsch!');
     end;
end
BRF=SV(1); %Auswahl einer Berechnungsspalte aus BM
SDF=SV(2); %Schalter dispflag = Anzeige "removed states"
SAK=SV(3); %Schalter Ablaufkontrolle = Teiergebnisanzeige
SOP=SV(4); %Schalter Options = fsolve mit Display & Diagnose

S=[1 1 1];	   % S=[Kdelta Km Kn] **** Schalter fuer Wichtungsfaktoren
RFV=[1 2 3];   % RFV=[1 2 3]	  **** Reihenfolgevektor von [Kdelta Km Kn]
   if sum(RFV)~=6 error('Reihenfolgevektor falsch'), end
BM=[2 1 3; 1 1 0; 0 1 0; 0 0 0];  % **** SPALTE - Berechnungsanzahl pro fsolve-Ruf
% Definition von Zaehlern und Nennern aller Bloecke
  n001=1;                          d001=[Tei 1];
  n002=1;                          d002=[TM 0];
  n003=1;                          d003=1;
  n004=1;                          d004=[TF 0];
  n005=1;                          d005=[TL 0];
% Angabe der Blockanzahl und Bildung eines unverbundenen diagonalen
% Zustandsmodelles des Systems aus den vorherigen Zaehler- und Nenner-
% Bloecken:
%nblocks=5;
%blkbuild;      % Wegem Globalvariablenzugriff nur in script-files!!!
% Unverbundenes Diagonalsystem erzeugen
[a,b,c,d]=tf2ss(n001,d001);  s0n=abs('n000'); s0d=abs('d000');
for t=2:5  HUS=fix(t/100); ZEV=fix(t/10); sk=[0 HUS ZEV-HUS*10 t-ZEV*10];
           s1=char(s0n+sk); s2=char(s0d+sk);
           [at,bt,ct,dt]=tf2ss(eval(s1),eval(s2));
           [a,b,c,d]=append(a,b,c,d,at,bt,ct,dt);
end;       %  !!!Algorithmus so fuer drei ASCII-Stellen (0...9)!!!
% Angabe der Verbindungen zwischen den Bloecken:
Q=[ 1  0  0
    2  1 -4
    3  2 -5
    4  3  0
    5  4  0];    in =[1];  ou=[3 4 2 5];
[ac,bc,cc,dc]=connect(a,b,c,d,Q,in,ou); % verbundenes System
[ap,bp,c,d]=minreal2(ac,bc,cc,dc,sqrt(eps),SDF); % Zustandseliminierung
[a,b]=c2d(ap,bp,T);                     % Z-Transformation
b(1,4)=0; d(4,4)=0;                     % Wiederherstellung einer sym.
                                        % Struktur=Anzahl Uebertraggsfktn
                                        % a,b,c,d: Unveraenderlicher Teil
PA=sum(S);		% Parameteranzahl
BA=sum(BM);		% Anfangswerte pro Variante (Spalte)
if PA~=BA(BRF) error('Falsche Berechnungsreihenfolge'), end
[i,j]=min(BM);		% i: Minima (0!) pro Spalte; j: zugehoerige Zeilennummer
SZ=j(BRF)-1;		% Schleifenzaehler fuer fsolve-Aufrufe
enr=1;			% Ergebnisnummer
erg=[];			% Anfangswert
for iS=1:SZ
 if iS~=1&SAK==1 erg, end % Ablaufkontrolle  ****
 TAZ=BM(iS,BRF);          % Transiente Anzahl
%Vor Umstellung von fsolve und options: AW=ones(1,TAZ); -->Nach Umstellung:
 AW=randn(1,TAZ);         % Variable Anfangswerteanzahl
%Vor Umstellung von fsolve und options:
 %q=fsolve('zust_aug1',AW',[0 0 0 0 1],[],a,b,c,d,erg,BRF,RFV,TAZ,iS);
%Nach Umstellung:
 if SOP
 options=optimset('Display','iter','LargeScale','on','Diagnostics','on',...
                      'TolFun',10^-30,'TolX',10^-20);
 else
 options=optimset('Display','off','LargeScale','on','TolFun',10^-30,'TolX',10^-20);
 end
 [q,z1,z2]=fsolve('zust_aug2',AW',options,a,b,c,d,erg,BRF,RFV,TAZ,iS,SDF);
 
 for j=1:TAZ
     erg(enr)=q(j);
     enr=enr+1;
 end
end
